/**
 * \file
 * This interface provides only one function at all.
 * indexUuid will generate a uuid to the current object.
 *
 * This one is not generic enough for an interface...I
 * don't want to rely on Uuid here....The Uuid stuff is more
 * an implementation of this interface.
 *
 * An implementation of indexable::index should always return
 * an instance of class Index / or an class that implements
 * the index interface. Uuid would then be a candidate for such
 * a class.
 *
 * \todo
 * Maybe merge hashable and indexable. Thus we might get an
 * easy way to exchange the hashing mechanism used for my
 * associative arrays.
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_INTERFACE_INDEXABLE_H__
#define __TR_INTERFACE_INDEXABLE_H__

#include "tr/interface.h"


typedef void * (* fptr_TR_getIndex)(void *);

TR_INTERFACE(TR_Indexable) {
	TR_IFID;
	fptr_TR_getIndex index;
};

/**
 * Get it's index from an indexable object.
 */
inline
void *
TR_getIndex(void * indexable)
{
	void * ret;
	TR_RETCALL(indexable, TR_Indexable, index, ret);
	return ret;
}

#endif // __TR_INTERFACE_INDEXABLE_H__

// vim: set ts=4 sw=4:
