/**
 * \file
 * Definition of the observer pattern implementation.
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __TR_INTERFACE_OBSERVER_H__
#define __TR_INTERFACE_OBSERVER_H__

#include "tr/interface.h"


typedef void (* fptr_TR_observerUpdate)(void *, void*);

/**
 * Interface structur for the observer pattern
 */
TR_INTERFACE(TR_Observer) {
	TR_IFID;
	fptr_TR_observerUpdate update;
};

/**
 * The implementation of update will be called by a
 * subject whenever it ments to inform the observer about
 * something
 */
inline
void
TR_observerUpdate(void * observer, void * subject)
{
	TR_CALL(observer, TR_Observer, update, subject);
}

#endif // __TR_INTERFACE_OBSERVER_H__

// vim: set ts=4 sw=4:
