/**
 * \file
 * Classes that implement this interface can be converted to an
 * array of bytes and created from an array of bytes if it has
 * the correct format. The format of th byte array is class
 * specific.
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_SERIALIZABLE_H__
#define __TR_SERIALIZABLE_H__

#include "tr/interface.h"


typedef void   (* fptr_serialize)(void *, unsigned char **, size_t *);
typedef void   (* fptr_unserialize)(void *, const unsigned char *, size_t);

TR_INTERFACE(TR_Serializable) {
	TR_IFID;
	fptr_serialize     serialize;
	fptr_unserialize   unserialize;
};

/**
 * Serialize the given instance to a byte array
 */
inline
void
TR_serialize(
		void           *  serializable,
		unsigned char ** serialized,
		size_t         *  nserialized)
{
	TR_CALL(serializable, TR_Serializable, serialize, serialized,
			nserialized);
}

/**
 * Unerialize the given instance to a byte array
 */
inline
void
TR_unserialize(
		void                * serializable,
		const unsigned char * serialized,
		size_t                nserialized)
{
	TR_CALL(serializable, TR_Serializable, unserialize, serialized,
			nserialized);
}

#endif // __TR_SERIALIZABLE_H__

// vim: set ts=4 sw=4:
