/**
 * \file
 * Definition of the subject pattern implementation.
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __SUBJECT_H__
#define __SUBJECT_H__

#include "tr/interface.h"


typedef void (* fptr_subjectAttach)(void *, void *);
typedef void (* fptr_subjectDetach)(void *, void *);
typedef void (* fptr_subjectNotify)(void *);

TR_INTERFACE(TR_Subject) {
	TR_IFID;
	fptr_subjectAttach attach;
	fptr_subjectDetach detach;
	fptr_subjectNotify notify;
};

/**
 * Attach an observer to a subject. After a successfull
 * call to this the subject will inform the observer about events.
 */
inline
void
TR_subjectAttach(void * subject, void * observer)
{
	TR_CALL(subject, TR_Subject, attach, observer);
}

/**
 * Detach an Observer from a Subject. After this no events
 * will be propagated from the subject to the observer anymore.
 */
inline
void
TR_subjectDetach(void * subject, void * observer)
{
	TR_CALL(subject, TR_Subject, detach, observer);
}

/**
 * Trigger the a notification of all attached observers.
 */
inline
void
TR_subjectNotify(void * subject)
{
	TR_CALL(subject, TR_Subject, notify);
}

#endif // __SUBJECT_H__

// vim: set ts=4 sw=4:
