/**
 * \file
 * A generic logger class and two extended classes, One that logs to
 * stderr and one that logs to the system syslog.
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_LOGGER_H__
#define __TR_LOGGER_H__

#include "trbase.h"

typedef enum {
	TR_LOGGER_DEBUG=0,
	TR_LOGGER_INFO,
	TR_LOGGER_NOTICE,
	TR_LOGGER_WARNING,
	TR_LOGGER_ERR,
	TR_LOGGER_CRIT,
	TR_LOGGER_ALERT,
	TR_LOGGER_EMERG
} TR_logger_level;

#include "tr/interface/logger.h"

extern const char * const TR_logger_level_str[];

TR_CLASS(TR_Logger) {
	TR_logger_level min_level;
};

TR_CLASS(TR_LoggerStderr) {
	TR_EXTENDS(TR_Logger);
};

TR_CLASS(TR_LoggerSyslog) {
	TR_EXTENDS(TR_Logger);
};

TR_INSTANCE_INIT(TR_Logger);
TR_INSTANCE_INIT(TR_LoggerStderr);
TR_INSTANCE_INIT(TR_LoggerSyslog);

TR_CLASSVARS_DECL(TR_Logger) {};
TR_CLASSVARS_DECL(TR_LoggerStderr) {};
TR_CLASSVARS_DECL(TR_LoggerSyslog) {};

extern TR_Logger TR_logger;

#endif // __TR_LOGGER_H__

// vim: set ts=4 sw=4:
