/**
 * \file
 * Common reusable definitions.
 * 
 * \todo All public symbols need to be prefixed to make name clashes
 *       more unlikely.
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "tr/math.h"

long
TR_euklidGcd(register long val1, register long val2)
{
    register long mod;

    if(val2 > val1) SWAP_XOR(val1, val2);

    mod = val1 % val2;
    while (mod != 0) {
        val1 = val2;
        val2 = mod;
        mod = val1 % val2;
    }

    return val2;
}

long
TR_lcm(register long val1, register long val2)
{   
    return val1 * val2 / TR_euklidGcd(val1, val2);
}

// vim: set ts=4 sw=4:
