/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define _POSIX_C_SOURCE 199309L

#include <time.h>
#include <limits.h>

#include "trbase.h"

unsigned long
TR_timerGet(TR_Timer this, unsigned long * missed)
{
	struct timespec    now;
	unsigned long long elapsed;
	unsigned long      sec_factor;
	long               nan_factor;
	unsigned long      _missed;

	clock_gettime(CLOCK_REALTIME, &now);

	switch (this->base) {
		case TR_TBASE_NAN:
			sec_factor = 1000 * 1000 * 1000;
			break;
		case TR_TBASE_MIC:
			sec_factor = 1000 * 1000;
			break;
		default:
		case TR_TBASE_MIL:
			sec_factor = 1000;
			break;
		case TR_TBASE_SEC:
			sec_factor = 1;
			break;
	}

	nan_factor = 1000 * 1000 * 1000 / sec_factor;
	elapsed = (now.tv_sec - this->sec) * sec_factor
		+ (now.tv_nsec - this->nsec) / nan_factor;

	_missed = elapsed / this->timeout;
	if (_missed) {
		this->sec  += _missed * this->timeout * sec_factor / nan_factor;
		this->nsec += _missed * this->timeout * sec_factor % nan_factor;
		if (missed) *missed = _missed;
	} else {
		if (missed) *missed = 0;
	}

	return this->timeout - (elapsed % this->timeout);
}

// vim: set ts=4 sw=4:
